// code by jph

#define _CRT_SECURE_NO_DEPRECATE  // no warnings when using std::c in vcpp compiler 14

#define TENSOR_SVD_NDEBUG

#define GLEW_INCLUDE

#include "../_util/_opengl.hpp"
#include "../_animated/aniMated.hpp"
#include "../_animated/aniAvatar.hpp"

aniAvatar*            ava;

opengl_light*         light_top;
opengl_light*         light_sid;
opengl_light*         light_fnt;

aniMated* ani=new aniMated();

int mode=0;
time launch;

tensor<double> crv;

meshgl cub("get/mesh/plane_460_tnv");

int main(int argc,char** argv) {
  opengl_recording_numel=600;
  opengl_recording_frame=1;     // take shot every x frames
  opengl_size.as(800,600);
  opengl_position.as(70,5);
  //opengl_clearcolor=opengl_color_black;
  opengl_main(argc,argv);
  
  ava=new aniAvatar();
  ava->pos(2,3)=3;
  ava->fva=tensor<double>(1,4).as( 0.3, 0.3, 0.3, 0.0);
  ava->fot=tensor<double>(1,4).as( 0.3, 0.3, 0.3, 0.0);
  aniMated::humanized=ava;
  
  tensor<double> trf;
  ani->msh=opengl_meshgl("get/mesh/sphere_224_tnv");
  trf=tensor<double>::eye(4);
  trf(0,0)=trf(1,1)=trf(2,2)=0.2;
  *ani->msh=ani->msh->map(trf);
  ani->msh->updateN3F();
  ani->shd=new shadow(ani->msh);

  
  light_top=new opengl_light();
  light_top->dif=tensor<double>(1,4).as( 0.5, 0.0, 0.0, 0.0); // TODO try without a=1.0
  light_top->spc=light_top->dif; 
  light_top->pos(0,3)=-10;
  light_top->pos(1,3)= 10;
  light_top->pos(2,3)=  5;
  
  light_sid=new opengl_light();
  light_sid->dif=tensor<double>(1,4).as( 0.0, 0.5, 0.0, 0.0); // TODO try without a=1.0
  light_sid->spc=light_sid->dif;
  light_sid->pos(0,3)=10;
  light_sid->pos(1,3)=0;
  light_sid->pos(2,3)= 0;
  
  light_fnt=new opengl_light();
  light_fnt->dif=tensor<double>(1,4).as( 0.0, 0.0, 0.5, 0.0); // TODO try without a=1.0
  light_fnt->spc=light_fnt->dif;
  light_fnt->pos(0,3)=0;
  light_fnt->pos(1,3)=-10;
  light_fnt->pos(2,3)=-5;

	tensor<double> lod("get/tensor/avatar.pos");
  if (lod.data)
    ava->pos=lod;
  ava->fva=10;
  
  trf=tensor<double>::eye(4);
  trf(0,0)=trf(1,1)=trf(2,2)=-5.5;
  trf(2,3)=-10;
  cub.format=GL_T2F_N3F_V3F;
  cub=cub.map(trf);
  cub.updateN3F();
  
  glutMainLoop();
  return 0;
}

void scene(void) {
  int c0,c1,c2;
  //aniMated* ani;
  aniMated::dt=min(opengl_rate.s(),1/12.5);

  ava->relax();
  ava->scene();
  ava->motion();
  
  for (int c0=1;c0<7;++c0)
    if (keyboard_down('0'+c0)) {
      char buf[80];
      sprintf(buf,"get/tensor/curve%i.arr",c0);
      crv=tensor<double>(buf);
      mode|=1;
      launch=opengl_time;
    }
  if (keyboard_down('\t'))
    mode=0;
  
  if (keyboard_down('z'))
    ava->pos=tensor<double>::eye(4);
  
  time delta=opengl_time-launch;
  tensor<double> itp=tensor<double>::vector(1).as(delta.s()/120);
  
  if (mode) {
    mode&=delta.s()<120;      
  
    for (int c0=0;c0<3;++c0)
      ani->pos(c0,3)=crv[c0].interp(itp);
  }
  
  //if ()  
  double prd;
  double ret=modf(delta.s()/10,&prd);
  prd*=10;
  ret*=10;
  //printf("%f %f\n",prd,ret);
  opengl_clearcolor=ret<0.3&&mode?opengl_color_white:opengl_color_black;
  
  opengl_camera=glI(ava->pos);
}

void display(void) {
  if (mode) {
    ani->draw_std();
    cub.draw();
  }
}

void shadowing(opengl_light* lgt) {
  if (mode)
    ani->draw_shadow(lgt->pos[3]);
}

void finish(void) {
  glPushAttrib(GL_ENABLE_BIT);
    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST);
    glColor3d( 1,0,0);
    if (mode) {
      glPointSize(3);
      glColor3d( 1,1,1);
      glBegin(GL_POINTS);
        glVertex3dv(ani->pos^12);
      glEnd();
      glColor3d( 1,0,0);
    } else {
      glBegin(GL_LINE_STRIP);
        glVertex3d(-1,-1,0);
        glVertex3d( 1,-1,0);
        glVertex3d( 1, 1,0);
        glVertex3d(-1, 1,0);
        glVertex3d(-1,-1,0);
      glEnd();
      glBegin(GL_LINE_STRIP);
        glVertex3d(-1,-1,-10);
        glVertex3d( 1,-1,-10);
        glVertex3d( 1, 1,-10);
        glVertex3d(-1, 1,-10);
        glVertex3d(-1,-1,-10);
      glEnd();
    }
  glPopAttrib();  
}

void shutdown(void) {
  exit(0);
}
