// code by jph

package speller;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import javax.swing.JComponent;

class State {
  StringBuffer myStringBuffer;
  String myString;
  
  State(StringBuffer myStringBuffer, String myString) {
    this.myStringBuffer = new StringBuffer(myStringBuffer);
    this.myString = myString;
  }
}

public class SpellerComponent extends JComponent {
  
  StringBuffer myStringBuffer = new StringBuffer();
  String myString = "";
  List<String> myFollower;
  int[] myInt;
  int count;
  int done;
  
  Stack<State> myStack = new Stack<State>();
  
  SpellerComponent() {
    register();
  }
  
  void register() {
    myFollower = Speller.follows(myString);
    myInt = new int[myFollower.size()];
    count = 0;
    undo = 0;
    done = (int) Math.round(done_min + (done_max - done_min) * Math.min(Math.max(0, myFollower.size() - 6) / 20.0, 1));
  }
  
  void append(String myChar) {
    if (myChar.isEmpty()) {
      myStack.push(new State(myStringBuffer, myString));
      myStringBuffer.append(myString + " ");
      myString = "";
    } else {
      myString = myString + myChar;
      if (Speller.follows(myString).size() == 2)
        append(Speller.follows(myString).get(0));
    }
    register();
  }
  
  void reduced() {
    while (Speller.follows(myString).size() == 2)
      myString = myString.substring(0, myString.length() - 1);
    register();
  }
  
  int done_min = 5;
  int done_max = 10;
  int timeout = 40;
  
  static Font fontToken = new Font(Font.DIALOG, Font.PLAIN, 40); // 60 
  static Font fontString = new Font(Font.DIALOG, Font.PLAIN, 80);
  static Font fontBuffer = new Font(Font.DIALOG, Font.PLAIN, 40);
  
  private Dimension myDimension = new Dimension(1, 1); // getSize()
  private Point myPoint = new Point(0, 0);
  
  public int flush(int done_min, int done_max, int timeout, double pix, double piy) {
    this.done_min = done_min;
    this.done_max = done_max;
    this.timeout = timeout;
    myPoint.x = (int) Math.round(pix * myDimension.width);
    myPoint.y = (int) Math.round(piy * myDimension.height);
    repaint();
    return myStringBuffer.length() + myString.length();
  }
  
  double[] angles() {
    double[] myDouble = new double[myFollower.size() + 1];
    for (int c0 = 1; c0 < myFollower.size(); ++c0)
      myDouble[c0] = c0 * Math.PI * 2 / myFollower.size();
    myDouble[myFollower.size()] = myDouble[0] + Math.PI * 2;
    return myDouble;
  }
  
  int undo = 0;
  
  public void paintComponent(Graphics myGraphics) {
    myDimension = getSize();
    int col = Math.min(Math.max(count - 2 * done, 0) * 255 / (timeout - 2 * done), 255);
    myGraphics.setColor(new Color(255, 255, 255 - col));
    myGraphics.fillRect(0, 0, myDimension.width, myDimension.height);
    
    buffer(myGraphics, myStringBuffer.toString());
    
    int ind = -1;
    double[] myDouble = angles();
    double rad = 0.8;
    myGraphics.setFont(fontToken);
    for (int c0 = 0; c0 < myInt.length; ++c0) {
      Point[] myArray = new Point[4];
      double a1 = myDouble[c0];
      double a2 = myDouble[c0 + 1];
      myArray[0] = map(0.5 * Math.sin(a1), 0.5 * Math.cos(a1));
      myArray[1] = map(3.5 * Math.sin(a1), 3.5 * Math.cos(a1));
      myArray[2] = map(3.5 * Math.sin(a2), 3.5 * Math.cos(a2));
      myArray[3] = map(0.5 * Math.sin(a2), 0.5 * Math.cos(a2));
      Polygon myPolygon = new Polygon();
      for (int c1 = 0; c1 < myArray.length; ++c1)
        myPolygon.addPoint(myArray[c1].x, myArray[c1].y);
      if (myPolygon.contains(myPoint))
        ind = c0;
      int cor = myInt[c0] * 255 / done;
      myGraphics.setColor(new Color(255 - cor, 255, 255 - cor));
      myGraphics.fillPolygon(myPolygon);
      
      myGraphics.setColor(Color.darkGray);
      myGraphics.drawPolygon(myPolygon);
      drawString(myGraphics, myFollower.get(c0), map(rad * Math.sin((a1 + a2) * 0.5), rad * Math.cos((a1 + a2) * 0.5)));
    }
    if (0 <= ind) {
      if (3 + 1 + 3 < myFollower.size())
        ++count;
      myInt[ind] += 2;
      for (int c0 = 0; c0 < myInt.length; ++c0)
        myInt[c0] = Math.min(Math.max(0, myInt[c0] - 1), done);
    }
    
    if (!myStack.isEmpty() && myString.isEmpty()) {
      Polygon myPolygon = new Polygon();
      int num = 30;
      double ang = 0;
      for (int c0 = 0; c0 <= num; ++c0) {
        rad = 0.20;
        Point myPoint = map(rad * Math.sin(ang), rad * Math.cos(ang));
        myPolygon.addPoint(myPoint.x, myPoint.y + myDimension.height / 9);
        ang += Math.PI * 2 / num;
      }
      undo = Math.min(Math.max(0, undo + (myPolygon.contains(myPoint) ? 1 : -1)), done);
      int cor = undo * 255 / done;
      myGraphics.setColor(new Color(255 - cor, 255, 255 - cor));
      myGraphics.fillPolygon(myPolygon);
      myGraphics.setColor(Color.darkGray);
      myGraphics.drawPolygon(myPolygon);
      if (undo == done) {
        State myState = myStack.pop();
        myStringBuffer = new StringBuffer(myState.myStringBuffer);
        myString = myState.myString;
        reduced();
        register();
        return;
      }
    }
    
    if (0 < myStringBuffer.length()) {
      String myString = myStringBuffer.toString();
      int beg = Math.max(0, myString.length() - 20);
      buffer(myGraphics, myString.substring(beg, myString.length()));
    }
    
    myGraphics.setFont(fontString);
    myGraphics.setColor(Color.red);
    Rectangle2D myRectangle2d = myGraphics.getFontMetrics().getStringBounds(myString, myGraphics);
    myGraphics.drawString(myString, myDimension.width / 2 - (int) myRectangle2d.getWidth() / 2, myDimension.height * 3 / 5 + (int) myRectangle2d.getHeight() / 3);
    
    myGraphics.setFont(fontToken);
    myGraphics.setColor(Color.darkGray);
    int width = 5;
    myGraphics.setColor(Color.red);
    myGraphics.fillArc(myPoint.x - width, myPoint.y - width, width * 2 + 1, width * 2 + 1, 0, 360);
    
    for (int c0 = 0; c0 < myInt.length; ++c0)
      if (myInt[c0] == done) {
        String myChar = myFollower.get(c0);
        if (myChar.equals(Speller.BACKSPACE)) {
          myString = myString.substring(0, myString.length() - 1);
          reduced();
        } else
          append(myFollower.get(c0));
        return;
      }
    if (timeout < count) {
      count = 0;
      List<String> myList = new ArrayList<String>();
      for (int c0 = ind - 3; c0 <= ind + 3; ++c0)
        myList.add(myFollower.get((c0 + myFollower.size()) % myFollower.size()));
      myFollower = myList;
      myInt = new int[myFollower.size()];
    }
//    myGraphics.drawString("" + done, 30, 30);
    
  }
  
  void buffer(Graphics myGraphics, String myString) {
    myGraphics.setFont(fontBuffer);
    myGraphics.setColor(Color.blue);
    Rectangle2D myRectangle2d = myGraphics.getFontMetrics().getStringBounds(myString, myGraphics);
    myGraphics.drawString(myString, myDimension.width * 5 / 7 - (int) myRectangle2d.getWidth(), myDimension.height * 3 / 7 - (int) myRectangle2d.getHeight() / 2);
  }
  
  Point map(double pix, double piy) {
    Point myPoint = new Point();
    double nr2 = Math.sqrt(pix * pix + piy * piy);
    double n = 3.;
    double nrn = Math.pow(Math.pow(Math.abs(pix), n) + Math.pow(Math.abs(piy), n), 1 / n) + 1e-5;
    pix *= nr2 / nrn;
    piy *= nr2 / nrn;
    myPoint.x = (int) Math.round((1 + pix) * 0.5 * myDimension.width);
    myPoint.y = (int) Math.round((1 - piy) * 0.5 * myDimension.height);
    return myPoint;
  }
  
  private void drawString(Graphics myGraphics, String myString, Point myPoint) {
    Rectangle2D myRectangle2d = myGraphics.getFontMetrics().getStringBounds(myString, myGraphics);
    myGraphics.drawString(myString, (int) Math.round(myPoint.x - myRectangle2d.getWidth() / 2), (int) Math.round(myPoint.y + 0.6 * myRectangle2d.getHeight() / 2));
  }
}
