function varargout=nuscalp3(val,chn,varargin)
% code by jph
% example 1 ---
%  nuscalp3(rand(1,41),1:41)
%
% example 2 ---
%  hnd=[];
%  for c1=1:20
%    hnd=nuscalp3(rand(1,41),1:41,hnd)
%    pause(1)
%  end

global nu
persistent model

if isempty(model)
  disp('remark: load head model')
  load 'res/head_model'
end

chn=reshape(chn,[1 length(chn)]);
eql=find(ismember(model.union,num2str(chn)));

if isempty(eql)
  disp('remark: computing interpolation')
  pow=4;
  num=size(model.vert,2);
  model.align=horzcat(model.align,zeros(length(chn),num));
  for c0=1:num
    dst=repmat(model.vert(:,c0)',[length(chn) 1])-model.plot(chn,:);
    wgt=1./(sqrt(sum(dst.*dst,2)).^pow+eps);
    model.align{end}(:,c0)=wgt/sum(wgt);
  end
  model.union=horzcat(model.union,num2str(chn));
  eql=length(model.union);
end

hnd=[];
if length(varargin)
  if isnumeric(varargin{1})
    hnd=varargin{1};
    varargin=varargin(2:end);
  end
end

col=reshape(val,[1 numel(val)])*model.align{eql(1)};

if ismember('normalize',varargin)
  bnd             =[-1 1];
  col([3225 3226])=[-1 1];
else
  bnd=[min(val) max(val)];
end

if isempty(hnd)
  hnd=trisurf(model.tria',model.vert(1,:),model.vert(2,:),model.vert(3,:),col);
  if ismember('alpha',varargin)
    alpha(hnd,model.alpha')
  end
  shading interp
  hold on  
  if ismember('plot',varargin)
    plot3(model.plot(chn,1),model.plot(chn,2),model.plot(chn,3),'k.');
  end  
  if ismember('text',varargin)
    fac=1.03;
    cnt=1;
    for c1=1:length(chn)
      c2=chn(c1);
      txt=text(model.plot(c2,1)*fac+sign(model.plot(c2,1))*0.00+0.00,...   % 0.00
               model.plot(c2,2)*fac+sign(model.plot(c2,2))*0.00+0.00,...   % 0.10
               model.plot(c2,3)*fac+sign(model.plot(c2,3))*0.00+0.00,model.text{c2}); % 0.17
      set(txt,'FontSize',7)
      bol= val(c1)-bnd(1) < 0.2 * ( bnd(2)-bnd(1) );
      set(txt,'Color',[0.2 0.2 0.2]+bol*0.4)
      set(txt,'HorizontalAlignment','center')
      % by default VerticalAlignment = middle   
    end
  end  
  axis equal  
  axis tight
  axis off
  hold off
else
  set(hnd,'FaceVertexCData',col');
end

if nargout
  varargout{1}=hnd;
end
